#source <(curl https://raw.githubusercontent.com/holgerbrandl/datautils/v1.8/R/spinr/spin_utils.sh 2>&1 2>/dev/null)


## configure paths
#python setup_w_cython.py install --prefix /home/brandl/bin/macs2

export PATH=/home/brandl/bin/macs2/bin:$PATH
export PYTHONPATH=/home/brandl/bin/macs2/lib:$PYTHONPATH
export PYTHONPATH=/home/brandl/bin/macs2/lib/python2.7/site-packages:$PYTHONPATH

#/home/brandl/bin/macs2/bin/macs2 -h


cs_bowtie_qc(){

echo '
devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.8/R/core_commons.R")
devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.8/R/ggplot_commons.R")

########################################################################################################################
#> # Bowtie2 Mapping Summary

#> Directory: `r getwd()`

logSuffix=".bowtie.log"
parseAlgnSummary <- function(alignSummary){
    #alignSummary="/lustre/projects/bioinfo/holger/projects/khan_chipseq_h1/alignments/H1L_Shield.bowtie.log"
    algnData <- readLines(alignSummary)

    data.frame(
        condition=trimEnd(basename(alignSummary), logSuffix),
        num_reads=as.numeric(str_split_fixed(algnData[1], " ", 2)[1]),
        mapping_efficiency=as.numeric(str_replace(str_split_fixed(algnData[6], " ", 2)[1], "%", "")),

        unique_mappers=as.numeric(str_split_fixed(str_trim(algnData[4]), " ", 2)[1]),
        unique_mapper_prop=str_match(algnData[4], "[(]([0-9.]*)[%)]*")  %>% subset(select=2) %>% as.numeric(),

        multi_mappers=as.numeric(str_split_fixed(str_trim(algnData[5]), " ", 2)[1]),
        multi_mappers_prop=str_match(algnData[5], "[(]([0-9.]*)[%)]*")  %>% subset(select=2) %>% as.numeric()
    )
}

algnSummary <- list.files(".", logSuffix, full.names=TRUE, recursive=T) %>% ldply(parseAlgnSummary)
write.delim(algnSummary, file="bowtie2_qc.txt")

scale_fill_discrete <- function (...){ scale_color_brewer(..., type = "seq", palette="Set1", "fill", na.value = "grey50") }

## make sure that mapping efficency is a ratio between 0 and 1
algnSummary %<>% mutate(mapping_efficiency=mapping_efficiency/100.)

#+ fig.height=nrow(algnSummary)
ggplot(algnSummary, aes(condition, mapping_efficiency)) +
    geom_bar(stat="identity") +
    coord_flip() +
    scale_y_continuous(labels=percent, limits=c(0,1))
    ggtitle("mapping efficiency")

ggplot(algnSummary, aes(condition, num_reads*mapping_efficiency)) +
    geom_bar(stat="identity") +
    xlab("# alignments") +
    coord_flip() +
    scale_y_continuous(labels=comma)
    ggtitle("alignment counts")

ggplot(algnSummary, aes(condition, num_reads)) +
    geom_bar(stat="identity") +
    coord_flip() +
    ggtitle("read counts") +
    scale_y_continuous(labels=comma)

ggplot(algnSummary, aes(condition, unique_mapper_prop)) +
    geom_bar(stat="identity") +
    coord_flip() +
    ggtitle("unique-mapper proportions") +
    scale_y_continuous(labels=percent) +
    ylim(0,100)


ggplot(algnSummary, aes(condition, multi_mappers_prop)) +
    geom_bar(stat="identity") +
    coord_flip() +
    ggtitle("multi-mapper proportions") +
    scale_y_continuous(labels=percent) +
    ylim(0,100)


#> ## Bam Correlation

#> Using simple genome binning, we can performa a simple clustering/corrlation analysis

## http://stackoverflow.com/questions/291813/recommended-way-to-embed-pdf-in-html
#+ results="asis"
cat("<embed src=\"./bc.pdf\" width=\"900px\" height=\"700px\">")

' | spinsnip "bowtie2_qc"

}
export -f cs_bowtie_qc



cs_lib_size(){
rm algn_counts.txt
for bamFile in $(ls $*); do
(
    echo "processing $bamFile"
    samtools flagstat $bamFile | head -n1 | cut -f1 -d' ' | sed -e 's/$/\t'$(basename $bamFile _mmf.bam)'/' >> algn_counts.txt
) &
done
wait

echo '
devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.22/R/core_commons.R")
devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.22/R/ggplot_commons.R")

libSizes <- read_tsv("algn_counts.txt", col_names=F) %>% set_names(c("library_size", "sample"))
libSizes %>% ggplot(aes(sample, library_size)) +
    geom_bar(stat="identity") +
    ggtitle("algn_counts") +
    coord_flip()

ggsave2()
' | R -q --vanilla
}
export -f cs_lib_size



## create sorted genome file for dba counting
cs_sync_order_chrominfo2bam(){

if [ $# -ne 2 ]; then
    echo "Usage: cs_sync_order_bam2bed <bam_file>  <chrom_info>" >&2 ; return;
fi

echo '
devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.8/R/core_commons.R")

bamFile=commandArgs(T)[1]
chromInfoFile=commandArgs(T)[2]

#bamFile="/projects/bioinfo/holger/projects/krause_chipseq/alignments_mmfilt/H3HA_Dome_mmf.bam";
#chromInfoFile="danRer7.chromInfo"

chrOrderBam <- readLines(pipe(paste("samtools view -H ", bamFile, " | cut -f2 | grep -F SN"))) %>% str_replace(fixed("SN:"), "")

chromInfo <- read.delim(chromInfoFile) %>% mutate(chrom=str_replace(chrom, "chr", ""))

#data.frame(chrom=chrOrderBam) %>%
#    left_join(chromInfo) %>%
#    ## make sure to always keep all chromsomes of the bam no matter if they are present in the chromInfo.
#    mutate(size=ifelse(is.na(size), -1, size)) %>%
#    write.delim(stdout(), header=F, quote=F)

read.delim(chromInfoFile) %>%
    mutate(chrom=str_replace(chrom, "chrM", "MT")) %>%
    mutate(chrom=str_replace(chrom, "chr", "")) %>%
    mutate(chrom=factor(as.character(chrom), levels=chrOrderBam)) %>% arrange(chrom) %>%
    filter(!is.na(chrom)) %>%
    write.delim(stdout(), header=F, quote=F)
' | Rscript --vanilla - $1 $2  2> /dev/null
}
export -f cs_sync_order_chrominfo2bam


cs_sync_order_bam2bed(){

if [ $# -ne 2 ]; then
    echo "Usage: cs_sync_order_bam2bed <bam_file>  <bed_file>" >&2 ; return;
fi

echo '
devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.8/R/core_commons.R")

bamFile=commandArgs(T)[1]
bedFile=commandArgs(T)[2]

options(warn = -1)

#bamFile="/projects/bioinfo/holger/projects/khan_chipseq_h1/alignments_mmfilt/H1M_Dome_mmf.bam"
#bedFile="zv9_ens79_genebodies.bed"

chrOrderBam <- readLines(pipe(paste("samtools view -H ", bamFile, " | cut -f2 | grep -F SN"))) %>% str_replace(fixed("SN:"), "")
#read.delim(bedFile, h=F) %>% slice(match(chrOrderBam,  as.character(V1))) %>%
read.delim(bedFile, h=F) %>% mutate(V1=factor(as.character(V1), levels=chrOrderBam)) %>% arrange(V1, V2) %>% write.delim(stdout(), header=F, quote=F)
' | Rscript --vanilla - $1 $2 2> /dev/null
}
export -f cs_sync_order_bam2bed

